@@ -8,10 +8,12 @@ __author__ = 'Arjan Molenaar'
 __version__ = '$revision$'
 __date__ = '$date$'
 
+from zope import component
 import gobject
 import gaphas
 from gaphor.misc import uniqueid
 from uml2 import Namespace, PackageableElement
+from event import DiagramItemCreateEvent
 
 class DiagramCanvas(gaphas.Canvas):
     """
