@@ -6,13 +6,33 @@ Factory for and registration of model elements.
 from zope import interface
 from zope import component
 from gaphor.misc import uniqueid, odict
-from gaphor.UML.element import Element
-from gaphor.UML.diagram import Diagram
+from gaphor.interfaces import IService, IEventFilter
 from gaphor.UML.interfaces import IElementCreateEvent, IElementDeleteEvent, \
                                   IFlushFactoryEvent, IModelFactoryEvent, \
-                                  IService
+                                  IElementChangeEvent, IElementEvent
 from gaphor.UML.event import ElementCreateEvent, ElementDeleteEvent, \
                              FlushFactoryEvent, ModelFactoryEvent
+from gaphor.UML.element import Element
+from gaphor.UML.diagram import Diagram
+
+
+class ElementChangedEventBlocker(object):
+    """
+    Blocks all events of type IElementChangeEvent.
+
+    This filter is placed when the the element factory flushes it's content.
+    """
+    component.adapts(IElementChangeEvent)
+    interface.implements(IEventFilter)
+
+    def __init__(self, event):
+        self._event = event
+
+    def filter(self):
+        """
+        Return the event or None if the event is not to be emitted.
+        """
+        return 'Blocked by ElementFactory.flush()'
 
 
 class ElementFactory(object):
@@ -33,19 +53,28 @@ class ElementFactory(object):
     def __init__(self):
         self._elements = odict.odict()
         self._observers = list()
+        self._app = None
 
     def init(self, app):
-        component.provideHandler(self._element_unlinked)
+        self._app = app
+        app.register_handler(self._element_notify)
+        app.register_handler(self._element_deleted)
 
     def shutdown(self):
+        # unregister after flush: the handler is needed to empty the _elements
+        if self._app:
+            self._app.unregister_handler(self._element_notify)
         self.flush()
+        if self._app:
+            self._app.unregister_handler(self._element_deleted)
 
     def create(self, type):
         """
         Create a new model element of type type.
         """
         obj = self.create_as(type, uniqueid.generate_id())
-        component.handle(ElementCreateEvent(self, obj))
+        if self._app:
+            self._app.handle(ElementCreateEvent(self, obj))
         return obj
 
     def create_as(self, type, id):
@@ -131,18 +160,26 @@ class ElementFactory(object):
         """
         Flush all elements (remove them from the factory).
         """
-        component.handle(FlushFactoryEvent(self))
+        app = self._app
+        if app:
+            app.handle(FlushFactoryEvent(self))
+            app.register_subscription_adapter(ElementChangedEventBlocker)
 
-        # First flush all diagrams:
-        for value in list(self.select(lambda e: isinstance(e, Diagram))):
-            # Make sure no updates happen while destroying the canvas
-            value.canvas.block_updates = True
-            value.unlink()
-
-        for key, value in self._elements.items():
-            #print 'ElementFactory: unlinking', value
-            #print 'references:', gc.get_referrers(value)
-            value.unlink()
+        try:
+            # First flush all diagrams:
+            for value in list(self.select(lambda e: isinstance(e, Diagram))):
+                # Make sure no updates happen while destroying the canvas
+                value.canvas.block_updates = True
+                value.unlink()
+                if not app:
+                    self._element_deleted(ElementDeleteEvent(self, value))
+            for key, value in self._elements.items():
+                value.unlink()
+                if not app:
+                    self._element_deleted(ElementDeleteEvent(self, value))
+        finally:
+            if app:
+                app.unregister_subscription_adapter(ElementChangedEventBlocker)
 
         assert len(self._elements) == 0, 'Still items in the factory: %s' % str(self._elements.values())
 
@@ -161,10 +198,20 @@ class ElementFactory(object):
         Send notification that a new model has been loaded by means of the
         ModelFactoryEvent event from gaphor.UML.event.
         """
-        component.handle(ModelFactoryEvent(self))
+        if self._app:
+            self._app.handle(ModelFactoryEvent(self))
 
-    @component.adapter(ElementDeleteEvent)
-    def _element_unlinked(self, event):
+    @component.adapter(IElementEvent)
+    def _element_notify(self, event):
+        """
+        Dispatch IElementEvent events to interested adapters registered
+        by (class, event).
+        """
+        if self._app:
+            self._app.handle(event.element, event)
+
+    @component.adapter(IElementDeleteEvent)
+    def _element_deleted(self, event):
         """
         Remove an element from the factory.
         """
