@@ -1,6 +1,7 @@
 
 import unittest
 from zope import component
+from gaphor.application import Application
 from gaphor.UML import *
 from gaphor.UML.interfaces import *
 import gc
@@ -26,10 +27,12 @@ component.provideHandler(handler)
 class ElementFactoryTestCase(unittest.TestCase):
 
     def setUp(self):
+        Application.init_components()
         self.factory = ElementFactory()
-        self.factory.init(None)
+        self.factory.init(Application)
 
     def tearDown(self):
+        self.factory.shutdown()
         del self.factory
         self.clearEvents()
 
@@ -43,7 +46,6 @@ class ElementFactoryTestCase(unittest.TestCase):
         ef = self.factory
 
         p = ef.create(Parameter)
-        wp = weakref.ref(p)
         assert len(ef.values()) == 1
 
     def testFlush(self):
@@ -61,6 +63,24 @@ class ElementFactoryTestCase(unittest.TestCase):
         assert wp() is None
         assert len(ef.values()) == 0
 
+
+    def testWithoutApplication(self):
+        ef = ElementFactory()
+
+        p = ef.create(Parameter)
+        assert ef.size() == 1, ef.size()
+
+        ef.flush()
+        assert ef.size() == 0, ef.size()
+
+        p = ef.create(Parameter)
+        assert ef.size() == 1, ef.size()
+
+        p.unlink()
+        # TODO: does not work
+        assert ef.size() == 1, ef.size()
+
+
     def testUnlink(self):
         ef = self.factory
         p = ef.create(Parameter)
@@ -80,7 +100,7 @@ class ElementFactoryTestCase(unittest.TestCase):
         p.unlink()
         del p
 
-        assert len(ef.values()) == 0
+        assert len(ef.values()) == 0, ef.values()
 
     def testCreateEvent(self):
         ef = self.factory
@@ -111,5 +131,26 @@ class ElementFactoryTestCase(unittest.TestCase):
         ef.flush()
         self.assertTrue(IFlushFactoryEvent.providedBy(last_event) )
 
+    def test_element_event(self):
+        ef = self.factory
 
-# vim:sw=4:et
+        events = []
+        @component.adapter(Class, IElementChangeEvent)
+        def handler(element, event, events=events):
+            assert element is event.element
+            events.append(event)
+
+        #Application.init_components()
+        Application.register_handler(handler)
+
+        try:
+            c = Class()
+            del events[:]
+            c.name = 'name'
+            assert len(events) == 1, events
+            assert events[0].new_value == 'name'
+        finally:
+            Application.unregister_handler(handler)
+
+
+# vim:sw=4:et:ai
