@@ -1,5 +1,8 @@
 """
-Adapters
+Connector adapters.
+
+To register connectors implemented in this module, it is imported in
+gaphor.adapter package.
 """
 
 from zope import interface, component
@@ -80,11 +83,15 @@ class AbstractConnect(object):
 
         self.connect_constraints(handle)
 
+        # TODO: Add handler that triggers on setting self.element.subject = None
+        # TODO: How should we disconnect this handler?
+        #       Maybe one global handler will suffice. Then the adapters can
+        #       be created on an as-needed basis.
+        
         # Set disconnect handler in the adapter, so it will also wotk if
         # connections are created programmatically.
         def _disconnect():
             self.disconnect(handle)
-            handle.disconnect = lambda: 0
         handle.disconnect = _disconnect
 
         return True
@@ -96,6 +103,7 @@ class AbstractConnect(object):
         """
         self.disconnect_constraints(handle)
         handle.connected_to = None
+        handle.disconnect = lambda: 0
 
 
     def connect_constraints(self, handle):
@@ -111,7 +119,7 @@ class AbstractConnect(object):
         lc = constraint.LineConstraint(line=(CanvasProjection(h1.pos, element),
                                              CanvasProjection(h2.pos, element)),
                                        point=CanvasProjection(handle.pos, line))
-        handle._connect_constraint = lc
+        handle.connection_data = lc
         element.canvas.solver.add_constraint(lc)
 
         handle.connected_to = element
@@ -123,11 +131,11 @@ class AbstractConnect(object):
         element it's attached to, by removing the constraints.
         """
         try:
-            if handle._connect_constraint:
-                self.line.canvas.solver.remove_constraint(handle._connect_constraint)
+            if handle.connection_data:
+                self.line.canvas.solver.remove_constraint(handle.connection_data)
         except AttributeError:
-            pass # No _connect_constraint property yet
-        handle._connect_constraint = None
+            pass # No connection_data property yet
+        handle.connection_data = None
 
 
     def _get_segment(self, handle):
@@ -199,7 +207,7 @@ class ElementConnect(AbstractConnect):
 class LineConnect(AbstractConnect):
     """
     Base class for connecting two lines to each other.
-    The line that is conencted to is called 'element', as in ElementConnect.
+    The line that is connected to is called 'element', as in ElementConnect.
 
     Once a line has been connected at both ends, and a model element is
     assigned to it, all items connectedt to this line (e.g. Comments)
@@ -295,7 +303,7 @@ class CommentLineElementConnect(ElementConnect):
         if handle.connected_to and opposite.connected_to:
             if isinstance(opposite.connected_to.subject, UML.Comment):
                 del opposite.connected_to.subject.annotatedElement[handle.connected_to.subject]
-            else:
+            elif opposite.connected_to.subject:
                 del handle.connected_to.subject.annotatedElement[opposite.connected_to.subject]
         super(CommentLineElementConnect, self).disconnect(handle)
 
@@ -369,6 +377,34 @@ class RelationshipConnect(ElementConnect):
 
     element_factory = inject('element_factory')
 
+    # relationships can sometimes by unary, i.e. association, flow
+    # override in deriving class to allow unary relationship
+    CAN_BE_UNARY = False
+
+    def glue(self, handle):
+        """
+        In addition to the normal check, both relationship ends may not be
+        connected to the same element. Same goes for subjects.
+        """
+        if not self.CAN_BE_UNARY:
+            opposite = self.line.opposite(handle)
+            line = self.line
+            element = self.element
+            connected_to = opposite.connected_to
+
+            # Element can not be a parent for itself.
+            if connected_to is element:
+                return None
+
+            # Same goes for subjects:
+            if connected_to and \
+                    (not (connected_to.subject or element.subject)) \
+                     and connected_to.subject is element.subject:
+                return None
+
+        return super(RelationshipConnect, self).glue(handle)
+
+
     def relationship(self, required_type, head, tail):
         """
         Find an existing relationship in the model that meets the
@@ -501,6 +537,8 @@ class RelationshipConnect(ElementConnect):
                 line = self.line
                 if line.subject:
                     self.connect_connected_items()
+            return True
+
 
     def disconnect(self, handle):
         """
@@ -527,24 +565,8 @@ class DependencyConnect(RelationshipConnect):
     component.adapts(items.NamedItem, items.DependencyItem)
 
     def glue(self, handle):
-        """
-        In addition to the normal check, both line ends may not be connected
-        to the same element. Same goes for subjects.
-        """
-        opposite = self.line.opposite(handle)
         line = self.line
         element = self.element
-        connected_to = opposite.connected_to
-
-        # Element can not be a parent for itself.
-        if connected_to is element:
-            return None
-
-        # Same goes for subjects:
-        if connected_to and \
-                (not (connected_to.subject or element.subject)) \
-                 and connected_to.subject is element.subject:
-            return None
 
         # Element should be a NamedElement
         if not element.subject or \
@@ -580,24 +602,8 @@ class ImplementationConnect(RelationshipConnect):
     component.adapts(items.NamedItem, items.ImplementationItem)
 
     def glue(self, handle):
-        """
-        In addition to the normal check, both line ends may not be connected
-        to the same element. Same goes for subjects.
-        """
-        opposite = self.line.opposite(handle)
         line = self.line
         element = self.element
-        connected_to = opposite.connected_to
-
-        # Element can not be a parent for itself.
-        if connected_to is element:
-            return None
-
-        # Same goes for subjects:
-        if connected_to and \
-                (not (connected_to.subject or element.subject)) \
-                 and connected_to.subject is element.subject:
-            return None
 
         # Element at the head should be an Interface
         if handle is line.head and \
@@ -624,31 +630,9 @@ class GeneralizationConnect(RelationshipConnect):
     """
     Connect Classifiers with a Generalization relationship.
     """
+    # FixMe: Both ends of the generalization should be of the same  type?
     component.adapts(items.ClassifierItem, items.GeneralizationItem)
 
-#    # FixMe: Both ends of the generalization should be of the same  type?
-    def glue(self, handle):
-        """
-        In addition to the normal check, both line ends may not be connected
-        to the same element. Same goes for subjects.
-        """
-        opposite = self.line.opposite(handle)
-        line = self.line
-        element = self.element
-        connected_to = opposite.connected_to
-
-        # Element can not be a parent for itself.
-        if connected_to is element:
-            return None
-
-        # Same goes for subjects:
-        if connected_to and \
-                (not (connected_to.subject or element.subject)) \
-                 and connected_to.subject is element.subject:
-            return None
-
-        return super(GeneralizationConnect, self).glue(handle)
-
     def connect_subject(self, handle):
         relation = self.relationship_or_new(UML.Generalization,
                     ('general', None),
@@ -665,24 +649,8 @@ class IncludeConnect(RelationshipConnect):
     component.adapts(items.UseCaseItem, items.IncludeItem)
 
     def glue(self, handle):
-        """
-        In addition to the normal check, both line ends may not be connected
-        to the same element. Same goes for subjects.
-        """
-        opposite = self.line.opposite(handle)
         line = self.line
         element = self.element
-        connected_to = opposite.connected_to
-        
-        # Element can not be a parent for itself.
-        if connected_to is element:
-            return None
-
-        # Same goes for subjects:
-        if connected_to and \
-                (not (connected_to.subject or element.subject)) \
-                 and connected_to.subject is element.subject:
-            return None
 
         if not (element.subject and isinstance(element.subject, UML.UseCase)):
             return None
@@ -705,25 +673,9 @@ class ExtendConnect(RelationshipConnect):
     component.adapts(items.UseCaseItem, items.ExtendItem)
 
     def glue(self, handle):
-        """
-        In addition to the normal check, both line ends may not be connected
-        to the same element. Same goes for subjects.
-        """
-        opposite = self.line.opposite(handle)
         line = self.line
         element = self.element
-        connected_to = opposite.connected_to
         
-        # Element can not be a parent for itself.
-        if connected_to is element:
-            return None
-
-        # Same goes for subjects:
-        if connected_to and \
-                (not (connected_to.subject or element.subject)) \
-                 and connected_to.subject is element.subject:
-            return None
-
         if not (element.subject and isinstance(element.subject, UML.UseCase)):
             return None
 
@@ -745,24 +697,8 @@ class ExtensionConnect(RelationshipConnect):
     component.adapts(items.ClassifierItem, items.ExtensionItem)
 
     def glue(self, handle):
-        """
-        In addition to the normal check, both line ends may not be connected
-        to the same element. Same goes for subjects.
-        """
-        opposite = self.line.opposite(handle)
         line = self.line
         element = self.element
-        connected_to = opposite.connected_to
-
-        # Element can not be a parent for itself.
-        if connected_to is element:
-            return None
-
-        # Same goes for subjects:
-        if connected_to and \
-                (not (connected_to.subject or element.subject)) \
-                 and connected_to.subject is element.subject:
-            return None
 
         # Element at the head should be a Class
         if handle is line.head and \
@@ -854,11 +790,10 @@ class AssociationConnect(RelationshipConnect):
     """
     component.adapts(items.ClassifierItem, items.AssociationItem)
 
+    CAN_BE_UNARY = True    # allow one classifier to be connected by association
+
     def glue(self, handle):
-        opposite = self.line.opposite(handle)
-        line = self.line
         element = self.element
-        connected_to = opposite.connected_to
 
         # Element should be a Classifier
         if not isinstance(element.subject, UML.Classifier):
@@ -952,16 +887,11 @@ class FlowConnect(RelationshipConnect):
     Connect FlowItem and Action/ObjectNode, initial/final nodes.
     """
 
+    CAN_BE_UNARY = True   # flow can connect same actions
+
     def glue(self, handle):
-        """
-        In addition to the normal check, both line ends may not be connected
-        to the same element. Same goes for subjects.
-        """
-        opposite = self.line.opposite(handle)
         line = self.line
-        element = self.element
-        subject = element.subject
-        connected_to = opposite.connected_to
+        subject = self.element.subject
 
         if handle is line.head and isinstance(subject, UML.FinalNode) \
            or handle is line.tail and isinstance(subject, UML.InitialNode):
@@ -1013,41 +943,14 @@ class FlowForkDecisionNodeConnect(FlowConnect):
     Decision/Merge node.
     """
     def glue(self, handle):
-        """
-        In addition to the normal check, one end should have at most one
-        edge (incoming or outgoing).
-        """
-        opposite = self.line.opposite(handle)
-        line = self.line
-        element = self.element
-        subject = element.subject
-        connected_to = opposite.connected_to
-
-        # Element can not connect back to itself
-        if connected_to is element:
+        # No cyclic connect is possible on a Flow/Decision node:
+        head, tail = self.line.head, self.line.tail
+        subject = self.element.subject
+        
+        if handle is head and tail.connected_to and tail.connected_to.subject is subject \
+           or handle is tail and head.connected_to and head.connected_to.subject is subject:
             return None
 
-        # Same goes for subjects:
-        if connected_to and \
-                (not (connected_to.subject or element.subject)) \
-                 and connected_to.subject is element.subject:
-            return None
-
-        # If one side of self.element has more than one edge, the
-        # type of node is determined (either join or fork).
-        #
-        # TODO: remove these restrictions and create a combined join/fork or
-        #       decision/merge node 
-        #if handle is line.head and len(subject.incoming) > 1 and len(subject.outgoing) > 0:
-        #    return None
-        #
-        #if handle is line.tail and len(subject.incoming) > 0 and len(subject.outgoing) > 1:
-        #    return None
-
-#        assert len(self.element.handles()) == 2, '%s: %d' % (self.element, len(self.element.handles()))
-#        h1, h2 = self.element.handles()
-#        return geometry.distance_line_point(h1.pos, h2.pos, (x, y))[1]
-
         return super(FlowForkDecisionNodeConnect, self).glue(handle)
 
     def combine_nodes(self):
@@ -1337,4 +1240,5 @@ class MessageLifelineConnect(ElementConnect):
 
 component.provideAdapter(MessageLifelineConnect)
 
+
 # vim:sw=4:et:ai
