@@ -29,7 +29,7 @@ class ConnectorTestCase(TestCase):
         adapter.connect(handle)
 
         assert handle.connected_to is comment
-        assert handle._connect_constraint is not None
+        assert handle.connection_data is not None
         assert not comment.subject.annotatedElement
 
         # Connecting two ends of the line to the same item is not allowed:
@@ -38,10 +38,10 @@ class ConnectorTestCase(TestCase):
         adapter.connect(handle)
 
         assert handle.connected_to is None, handle.connected_to
-        assert not hasattr(handle,'_connect_constraint')
+        #assert not hasattr(handle,'connection_constraint')
         assert not comment.subject.annotatedElement, comment.subject.annotatedElement
 
-        #print '# now connect the actor'
+        # now connect the actor
 
         adapter = component.queryMultiAdapter((actor, line), IConnect)
 
@@ -49,7 +49,7 @@ class ConnectorTestCase(TestCase):
         adapter.connect(handle)
 
         assert handle.connected_to is actor
-        assert handle._connect_constraint is not None
+        assert handle.connection_data is not None
         assert len(comment.subject.annotatedElement) == 1, comment.subject.annotatedElement
         assert actor.subject in comment.subject.annotatedElement, comment.subject.annotatedElement
 
@@ -61,7 +61,7 @@ class ConnectorTestCase(TestCase):
         adapter.connect(handle)
 
         assert handle.connected_to is actor2
-        assert handle._connect_constraint is not None
+        assert handle.connection_data is not None
         assert len(comment.subject.annotatedElement) == 1, comment.subject.annotatedElement
         assert actor2.subject in comment.subject.annotatedElement, comment.subject.annotatedElement
 
@@ -70,10 +70,84 @@ class ConnectorTestCase(TestCase):
         adapter.disconnect(handle)
 
         assert handle.connected_to is None, handle.connected_to
-        assert handle._connect_constraint is None
+        assert handle.connection_data is None
         assert len(comment.subject.annotatedElement) == 0, comment.subject.annotatedElement
         assert not actor2.subject in comment.subject.annotatedElement, comment.subject.annotatedElement
 
+        adapter = component.queryMultiAdapter((comment, line), IConnect)
+
+        handle = line.head
+        adapter.disconnect(handle)
+        
+
+    def test_commentline_class(self):
+        """
+        Connect a CommentLine to a class and unlink the commentLine
+        afterwards.
+        """
+        clazz = self.create(items.ClassItem, UML.Class)
+        comment = self.create(items.CommentItem, UML.Comment)
+        line = self.create(items.CommentLineItem)
+
+        adapter = component.queryMultiAdapter((comment, line), IConnect)
+        handle = line.head
+        adapter.connect(handle)
+
+        adapter = component.queryMultiAdapter((clazz, line), IConnect)
+        handle = line.tail
+        adapter.connect(handle)
+
+        assert clazz.subject in comment.subject.annotatedElement
+        assert comment.subject in clazz.subject.ownedComment
+
+        line.unlink()
+
+        assert not comment.subject.annotatedElement
+        assert not clazz.subject.ownedComment
+
+
+    def test_commentline_relationship_unlink(self):
+        """
+        Connect a CommentLine to a relationship item.
+        Removing the relationship should work.
+
+        Demonstrates defect #103.
+        """
+        clazz1 = self.create(items.ClassItem, UML.Class)
+        clazz2 = self.create(items.ClassItem, UML.Class)
+        gen = self.create(items.GeneralizationItem)
+
+        adapter = component.queryMultiAdapter((clazz1, gen), IConnect)
+        handle = gen.head
+        adapter.connect(handle)
+
+        adapter = component.queryMultiAdapter((clazz2, gen), IConnect)
+        handle = gen.tail
+        adapter.connect(handle)
+
+        assert gen.subject
+
+        # And now the comment:
+
+        comment = self.create(items.CommentItem, UML.Comment)
+        line = self.create(items.CommentLineItem)
+
+        adapter = component.queryMultiAdapter((comment, line), IConnect)
+        handle = line.head
+        adapter.connect(handle)
+
+        adapter = component.queryMultiAdapter((gen, line), IConnect)
+        handle = line.tail
+        adapter.connect(handle)
+
+        assert gen.subject in comment.subject.annotatedElement
+        assert comment.subject in gen.subject.ownedComment
+
+        gen.unlink()
+
+        assert not comment.subject.annotatedElement
+        assert not gen.subject
+
 
     def test_commentline_association(self):
         """
@@ -117,7 +191,7 @@ class ConnectorTestCase(TestCase):
         adapter.connect(handle)
 
         assert handle.connected_to is assoc
-        assert handle._connect_constraint is not None
+        assert handle.connection_data is not None
         assert not comment.subject.annotatedElement
 
         # Connecting two ends of the line to the same item is not allowed:
@@ -126,7 +200,6 @@ class ConnectorTestCase(TestCase):
         adapter.connect(handle)
 
         assert handle.connected_to is None
-        assert not hasattr(handle,'_connect_constraint')
         assert not comment.subject.annotatedElement, comment.subject.annotatedElement
 
         # now connect the comment
@@ -137,7 +210,7 @@ class ConnectorTestCase(TestCase):
         adapter.connect(handle)
 
         assert handle.connected_to is comment
-        assert handle._connect_constraint is not None
+        assert handle.connection_data is not None
         assert len(comment.subject.annotatedElement) == 1, comment.subject.annotatedElement
         assert assoc.subject in comment.subject.annotatedElement, comment.subject.annotatedElement
 
@@ -146,7 +219,7 @@ class ConnectorTestCase(TestCase):
         adapter.disconnect(handle)
 
         assert handle.connected_to is None, handle.connected_to
-        assert handle._connect_constraint is None
+        assert handle.connection_data is None
         assert len(comment.subject.annotatedElement) == 0, comment.subject.annotatedElement
         assert not assoc.subject in comment.subject.annotatedElement, comment.subject.annotatedElement
 
@@ -177,7 +250,7 @@ class ConnectorTestCase(TestCase):
         adapter.connect(handle)
 
         assert handle.connected_to is comment
-        assert handle._connect_constraint is not None
+        assert handle.connection_data is not None
         assert len(comment.subject.annotatedElement) == 0, comment.subject.annotatedElement
 
         # connect opposite end to the association:
@@ -187,7 +260,7 @@ class ConnectorTestCase(TestCase):
         adapter.connect(handle)
 
         assert handle.connected_to is assoc
-        assert handle._connect_constraint is not None
+        assert handle.connection_data is not None
         assert len(comment.subject.annotatedElement) == 0, comment.subject.annotatedElement
         assert assoc.subject is None
 
@@ -198,7 +271,7 @@ class ConnectorTestCase(TestCase):
         adapter.connect(handle)
 
         assert handle.connected_to is c1
-        assert handle._connect_constraint is not None
+        assert handle.connection_data is not None
         assert len(comment.subject.annotatedElement) == 0, comment.subject.annotatedElement
         assert assoc.subject is None
 
@@ -485,8 +558,8 @@ class ConnectorTestCase(TestCase):
         a2 = self.create(items.ActionItem, UML.Action)
         o1 = self.create(items.ObjectNodeItem, UML.ObjectNode)
 
-        # diagram, two actions and object node with literal specification, flow has no subject
-        assert len(self.element_factory.lselect()) == 5, self.element_factory.lselect()
+        # diagram, two actions and object node, flow has no subject
+        assert len(self.element_factory.lselect()) == 4, self.element_factory.lselect()
 
         # Connect between two actions (ControlFlow)
         adapter = component.queryMultiAdapter((a1, flow), IConnect)
@@ -637,6 +710,10 @@ class ConnectorTestCase(TestCase):
         assert type(f1.subject) is forkNodeClass, f1.subject
 
         # flow4 will force the forknode to become a combined node:
+        # Connecting line this:
+        #        head  tail|--flow2-->[ a2 ]
+        # [ a1 ] --flow1-->|
+        # [ a4 ] --flow4-->|--flow3-->[ a3 ]
 
         adapter = component.queryMultiAdapter((f1, flow4), IConnect)
         assert adapter
