@@ -1,5 +1,5 @@
 """
-Artifact item
+Artifact item.
 """
 
 from gaphor import UML
@@ -10,6 +10,10 @@ class ArtifactItem(ClassifierItem):
     __uml__  = UML.Artifact
     __icon__ = True
 
+    __style__ = {
+            'name-padding': (10, 25, 10, 10),
+    }
+
     ICON_HEIGHT = 20
 
     def __init__(self, id=None):
@@ -20,8 +24,8 @@ class ArtifactItem(ClassifierItem):
         self.drawing_style = self.DRAW_COMPARTMENT_ICON
         self._line = []
         
-    def update(self, context):
-        super(ArtifactItem, self).update(context)
+    def pre_update_compartment_icon(self, context):
+        super(ArtifactItem, self).pre_update_compartment_icon(context)
         w = self.ICON_WIDTH
         h = self.ICON_HEIGHT
         ix, iy = self.get_icon_pos()
@@ -35,9 +39,12 @@ class ArtifactItem(ClassifierItem):
                 (ix + w, iy + h),
                 (ix + w, iy + ear))
 
+
     def draw_compartment_icon(self, context):
         cr = context.cairo
         cr.save()
+        self.draw_compartment(context)
+        cr.restore()
 
         # draw icon
         w = self.ICON_WIDTH
@@ -50,7 +57,6 @@ class ArtifactItem(ClassifierItem):
             cr.line_to(x, y)
         cr.stroke()
 
-        cr.restore()
-        self.draw_compartment(context)
+
 
 # vim:sw=4:et
