@@ -3,9 +3,12 @@
 
 import gtk
 import gaphas
+from diagrampainters import DefaultPainter
+
 
 class DiagramView(gaphas.GtkView):
-    """Displays a diagram (canvas) in a widget.
+    """
+    Displays a diagram (canvas) in a widget.
 
     See also: DiagramTab
     """
@@ -21,14 +24,11 @@ class DiagramView(gaphas.GtkView):
         super(DiagramView, self).__init__(diagram and diagram.canvas)
         self.diagram = diagram
         self.canvas = diagram.canvas
+        self.painter = DefaultPainter()
 
         # Drop
         self.drag_dest_set (gtk.DEST_DEFAULT_ALL, DiagramView.DND_TARGETS,
                             gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_LINK)
 
-#    def set_tool(self, tool):
-#        print 'setting tool', tool
-#        resource('MainWindow').get_action_pool().executeself.tool.grab(tool)
-#        pass
 
 # vim: sw=4:et
