@@ -9,7 +9,7 @@ from gaphor.interfaces import IActionProvider
 from interfaces import IUIComponent
 
 from gaphor import UML
-from gaphor.core import _, inject, action, radio_action, build_action_group, transactional
+from gaphor.core import _, inject, action, toggle_action, build_action_group, transactional
 from namespace import NamespaceModel, NamespaceView
 from diagramtab import DiagramTab
 from toolbox import Toolbox
@@ -20,6 +20,7 @@ from toplevelwindow import ToplevelWindow
 
 from interfaces import IDiagramSelectionChange
 from gaphor.interfaces import IServiceEvent, IActionExecutedEvent
+from gaphor.UML.event import ModelFactoryEvent
 from event import DiagramSelectionChange
 from gaphor.application import Application
 from gaphor.services.filemanager import FileManagerStateChanged
@@ -67,6 +68,8 @@ class MainWindow(ToplevelWindow):
             <menuitem action="tree-view-delete-diagram" />
             <menuitem action="tree-view-delete-package" />
             <separator />
+            <menuitem action="reset-tool-after-create" />
+            <separator />
             <placeholder name="primary" />
             <placeholder name="secondary" />
             <placeholder name="ternary" />
@@ -126,6 +129,7 @@ class MainWindow(ToplevelWindow):
             a.set_property('hide-if-empty', False)
             self.action_group.add_action(a)
         self._tab_ui_settings = None
+        self.action_group.get_action('reset-tool-after-create').set_active(self.properties.get('reset-tool-after-create', True))
 
     tree_model = property(lambda s: s.tree_view.get_model())
 
@@ -267,6 +271,7 @@ class MainWindow(ToplevelWindow):
         self.window.connect('destroy', self._on_window_destroy)
 
         Application.register_handler(self._action_executed)
+        Application.register_handler(self._new_model_content)
 
     def _update_toolbox(self, action_group):
         """
@@ -355,6 +360,32 @@ class MainWindow(ToplevelWindow):
 
     # Signal callbacks:
 
+    @component.adapter(ModelFactoryEvent)
+    def _new_model_content(self, event):
+        """
+        Open the toplevel element and load toplevel diagrams.
+        """
+        # Expand all root elements:
+        self.tree_view.expand_root_nodes()
+
+        # Open all diagrams under root node.
+        # TODO: move this! This is generic code.
+        # TODO: Make handlers for ModelFactoryEvent from within the GUI obj
+        model = self.tree_model
+        try:
+            iter = model.get_iter((0,))
+        except ValueError:
+            # no data
+            pass
+        else:
+            if model.iter_has_child(iter):
+                iter = model.iter_children(iter)
+                while iter:
+                    e = model.get_value(iter, 0)
+                    if isinstance(e, UML.Diagram):
+                        self.show_diagram(e)
+                    iter = model.iter_next(iter)
+    
     @component.adapter(FileManagerStateChanged)
     def _action_executed(self, event):
         # We're only interested in file operations
@@ -378,6 +409,7 @@ class MainWindow(ToplevelWindow):
         if gobject.main_depth() > 0:
             gtk.main_quit()
         Application.unregister_handler(self._action_executed)
+        Application.unregister_handler(self._new_model_content)
 
     def _on_tab_destroy(self, widget):
         tab = self.notebook_map[widget]
@@ -452,7 +484,7 @@ class MainWindow(ToplevelWindow):
         self._update_toolbox(tab.toolbox.action_group)
 
         # Make sure everyone knows the selection has changed.
-        component.handle(DiagramSelectionChange(tab.view, tab.view.focused_item, tab.view.selected_items))
+        Application.handle(DiagramSelectionChange(tab.view, tab.view.focused_item, tab.view.selected_items))
 
     def _on_window_size_allocate(self, window, allocation):
         """
@@ -469,6 +501,9 @@ class MainWindow(ToplevelWindow):
     @action(name='file-quit', stock_id='gtk-quit')
     def quit(self):
         self.ask_to_close() and gtk.main_quit()
+        self._tree_view.get_model().close()
+        Application.unregister_handler(self._action_executed)
+        Application.unregister_handler(self._new_model_content)
 
     @action(name='tree-view-open', label='_Open')
     def tree_view_open_selected(self):
@@ -515,8 +550,7 @@ class MainWindow(ToplevelWindow):
                               gtk.BUTTONS_YES_NO,
                               'Do you really want to delete diagram %s?\n\n'
                               'This will possibly delete diagram items\n'
-                              'that are not shown in other diagrams.\n'
-                              'This operation is not undoable!' \
+                              'that are not shown in other diagrams.'
                               % (diagram.name or '<None>'))
         if (m.run() == gtk.RESPONSE_YES):
             for i in reversed(diagram.canvas.get_all_items()):
@@ -554,6 +588,10 @@ class MainWindow(ToplevelWindow):
         self._tree_view.get_model().refresh()
 
 
+    @toggle_action(name='reset-tool-after-create', label=_('_Reset tool'), active=False)
+    def reset_tool_after_create(self, active):
+        self.properties.set('reset-tool-after-create', active)
+
 gtk.accel_map_add_filter('gaphor')
 
 
