@@ -5,7 +5,9 @@ Toolbox.
 import gobject
 import gtk
 
-from gaphor import resource
+from gaphor.core import inject
+
+from wrapbox import Wrapbox
 
 class Toolbox(gtk.VBox):
     """
@@ -17,6 +19,10 @@ class Toolbox(gtk.VBox):
 
     The toolbox is generated based on a definition with the form:
     ('name', ('boxAction1', 'boxAction2',...), 'name2', ('BoxActionN',...))
+
+    1 Create action pool for placement actions
+    2 Create gtk.RadioButtons for each item.
+    3 connect to action
     """
 
     __gsignals__ = {
@@ -24,15 +30,18 @@ class Toolbox(gtk.VBox):
                     gobject.TYPE_NONE, (gobject.TYPE_STRING, gobject.TYPE_INT))
     }
 
-    def __init__(self, menu_factory, toolboxdef):
+    properties = inject('properties')
+
+    def __init__(self, toolboxdef):
         """
         Create a new Toolbox instance. Wrapbox objects are generated
         using the menu_factory and based on the toolboxdef definition.
         """
         self.__gobject_init__()
-        self.menu_factory = menu_factory
         self.toolboxdef = toolboxdef
-        self.boxes = []
+        #self.boxes = []
+        self.buttons = []
+        self._construct()
 
     def on_wrapbox_decorator_toggled(self, button, content):
         """
@@ -51,8 +60,8 @@ class Toolbox(gtk.VBox):
             arrow.set(gtk.ARROW_RIGHT, gtk.SHADOW_IN)
             self.emit('toggled', button.toggle_id, False)
         # Save the property:
-        resource.set('ui.toolbox.%s' % button.toggle_id,
-                     content.get_property('visible'), persistent=True)
+        self.properties.set('ui.toolbox.%s' % button.toggle_id,
+                            content.get_property('visible'))
 
     def make_wrapbox_decorator(self, title, content):
         """
@@ -84,57 +93,51 @@ class Toolbox(gtk.VBox):
 
         button.connect('clicked', self.on_wrapbox_decorator_toggled, content)
 
-        #AJM: vbox.pack_start(content, True, True)
-        
         vbox.label = label
         vbox.content = content
 
-        expanded = resource('ui.toolbox.%s' % button.toggle_id, False)
+        expanded = self.properties.get('ui.toolbox.%s' % button.toggle_id, False)
         content.set_property('visible', not expanded)
         self.on_wrapbox_decorator_toggled(button, content)
 
         return vbox
 
-    def new_make_wrapbox_decorator(self, title, content):
-        """
-        Create a gtk.VBox with in the top compartment a label that can be
-        clicked to show/hide the lower compartment.
-        """
-        expander = gtk.Expander()
-        hbox = gtk.HBox()
-        expander.set_label_widget(hbox)
+    def toolbox_button(self, action_name, stock_id,
+                       icon_size=gtk.ICON_SIZE_LARGE_TOOLBAR):
+        button = gtk.ToggleButton()
+        if stock_id:
+            icon = gtk.Image()
+            icon.set_from_stock(stock_id, icon_size)
+            button.add(icon)
+            icon.show()
+        else:
+            button.props.label = action_name
+        button.action_name = action_name
+        return button
 
-        label = gtk.Label(title)
-        hbox.pack_start(label, expand=False, fill=False)
-
-        sep = gtk.HSeparator()
-        hbox.pack_start(sep, expand=True, fill=True)
-        hbox.set_spacing(3)
-
-        expander.add(content)
-        
-        expanded = resource('ui.toolbox.%s' % title.replace(' ', '-').lower(), False)
-        expander.set_expanded(expanded)
-
-        expander.show_all()
-        return expander
-
-    def construct(self):
+    def _construct(self):
 
         self.set_border_width(3)
-        vbox = self
 
-        wrapbox_groups = { }
+        self.tooltips = gtk.Tooltips()
+
         for title, items in self.toolboxdef:
-            wrapbox = self.menu_factory.create_wrapbox(items,
-                                                       groups=wrapbox_groups)
+            wrapbox = Wrapbox()
+            action = None
+            for action_name, label, stock_id in items:
+                button = self.toolbox_button(action_name, stock_id)
+                if label:
+                    self.tooltips.set_tip(button, label)
+                self.buttons.append(button)
+                wrapbox.add(button)
+                button.show()
             if title:
                 wrapbox_dec = self.make_wrapbox_decorator(title, wrapbox)
-                vbox.pack_start(wrapbox_dec, expand=False)
-                self.boxes.append(wrapbox_dec)
+                self.pack_start(wrapbox_dec, expand=False)
             else:
-                vbox.pack_start(wrapbox, expand=False)
+                self.pack_start(wrapbox, expand=False)
                 wrapbox.show()
-                self.boxes.append(wrapbox)
+
+        self.tooltips.enable()
 
 # vim:sw=4:et:ai
